==> 0.4.0%2B1.tar.gz: lib/scanner/snowm.dart <==
part of snowm_scanner;

class _SnowMScanner {
  static const MethodChannel _channel = const MethodChannel('snowm_scanner');
  int _beaconScannerCount = 0;
  int _stateScannerCount = 0;
  Map<String, StreamController<List<SnowMBeacon>>> _beaconStreamPool = {};
  Map<String, StreamController<BluetoothState>> _stateStreamPool = {};
  Map<String, Map<String, SnowMBeacon>> _cachedBeacons = {};

  bool _enableMqtt = false;
  int _timeInterval = 600000;

  _SnowMScanner() {
    _channel.setMethodCallHandler((call) {
      String method = call.method;
      if (method.contains("scanBeacons") &&
          _beaconStreamPool.containsKey(call.method)) {
        List<SnowMBeacon> beacons = call.arguments["beacons"]
            .map<SnowMBeacon>((b) => SnowMBeacon._fromMap(b))
            .toList();
        _beaconStreamPool[call.method].add(beacons);
      } else if (method.contains("bluetoothStateListener")) {
        _stateStreamPool[call.method]
            .add(getBluetoothStateFromKey(call.arguments));
      }
      return;
    });
  }

  void configure({
    /// Should scanner send messages to mqtt protocol. Default [false]
    bool enableMqtt = false,

    /// Time interval[Milliseconds] to send data to IOT cloud.Default is 10 minutes
    /// Valid only if [enableMqtt] is true.
    int timeInterval = 600000,
  }) {
    assert(timeInterval >= 60000);
    this._enableMqtt = enableMqtt;
    this._timeInterval = timeInterval;
  }

  Stream<BluetoothState> getBluetoothStateStream() {
    String method = "bluetoothStateListener#${++_stateScannerCount}";
    _stateStreamPool[method] =
        _getNewBluetoothStateStreamController<BluetoothState>(method);
    _channel.invokeMethod(method);
    return _stateStreamPool[method].stream;
  }

  Future<BluetoothPermissionState> getPermissionState() async {
    return getBluetoothPermissionStateFromKey(
        await _channel.invokeMethod<String>("permissionState"));
  }

  // TODO: Make it asyncronous
  void requestPermission() {
    _channel.invokeMethod<String>("requestPermission");
  }

  Future<BluetoothState> getBluetoothState() async {
    return getBluetoothStateFromKey(
        await _channel.invokeMethod<String>("bluetoothState"));
  }

  Future<void> scanBeaconsInBackground(
    List<String> uuids, {

    /// [Android] Title of the foreground scanner notification title
    String title = "SnowM Scanner",

    /// [Android] Title of the foreground scanner notification body
    String body = "We are scanning beacons currently.",

    /// Geofence parameters, if this is passed the scanner will only scan when the user gets inside the geofence.
    Geofence geofence,

    /// Custom payload that can be sent to scanner which will be delivered along with the scanned beacons
    Map<String, dynamic> customData = const {},
  }) async {
    assert(uuids.length > 0);
    await _channel.invokeMethod('scanBeaconBackground', {
      "uuids": uuids.map((e) => e.toUpperCase()).toList(),
      "geofenceEnabled": geofence != null,
      "geofence": geofence?.toMap(),
      "customData": customData,
      "title": title,
      "body": body,
    });
  }

  Future<void> stopBackgroudScan() async {
    await _channel.invokeMethod('stopBackgroundScan');
  }

  Stream<List<SnowMBeacon>> scanBeacons({
    /// List of proximity uuid of a beacon to be scanned
    List<String> uuids,

    /// Milliseconds for which the beacons should be cached. Default value is 60 seconds.
    int cacheBeaconsFor = 10000,
  }) {
    String method = "scanBeacons#${++_beaconScannerCount}";
    _beaconStreamPool[method] =
        _getNewBeaconStreamController<List<SnowMBeacon>>(method);
    _channel.invokeMethod(method, {
      "uuids": uuids.map((e) => e.toUpperCase()).toList(),
      "enableMqtt": _enableMqtt,
      "timeInterval": _timeInterval
    });
    Map<String, SnowMBeacon> cachedBeacons = _cachedBeacons[method] ?? {};
    return _beaconStreamPool[method].stream.map((beacons) {
      int currentTime = DateTime.now().millisecondsSinceEpoch;
      beacons = beacons.map((e) {
        e.._detectedTime = currentTime;
        cachedBeacons[e.uuid] = e;
        return e;
      }).toList();
      List<SnowMBeacon> freshBeacons = cachedBeacons.values
          .where((b) => (currentTime - b._detectedTime) < cacheBeaconsFor)
          .toList();
      cachedBeacons.clear();
      freshBeacons.forEach((b) {
        cachedBeacons[b.uuid] = b;
      });
      return freshBeacons;
    });
  }

  StreamController<T> _getNewBeaconStreamController<T>(String identifier) {
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelStream", {
        "methodName": identifier,
      });
      _beaconStreamPool.remove(identifier);
    };
    return controller;
  }

  StreamController<T> _getNewBluetoothStateStreamController<T>(
      String identifier) {
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelBluetoothStateListener", {
        "methodName": identifier,
      });
      _beaconStreamPool.remove(identifier);
    };
    return controller;
  }
}

final snowmScanner = _SnowMScanner();

==> 0.7.0%2B1.tar.gz: lib/scanner/snowm.dart <==
part of snowm_scanner;

class _SnowMScanner {
  static const MethodChannel _channel = const MethodChannel('snowm_scanner');
  int _beaconScannerCount = 0;
  int _stateScannerCount = 0;
  Map<String, StreamController<List<SnowMBeacon>>> _beaconStreamPool = {};
  Map<String, StreamController<BluetoothState>> _stateStreamPool = {};
  Map<String, Map<String, SnowMBeacon>> _cachedBeacons = {};

  bool _enableMqtt = false;
  int _timeInterval = 600000;

  _SnowMScanner() {
    _channel.setMethodCallHandler((call) {
      String method = call.method;
      if (method.contains("scanBeacons") &&
          _beaconStreamPool.containsKey(call.method)) {
        List<SnowMBeacon> beacons = call.arguments["beacons"]
            .map<SnowMBeacon>((b) => SnowMBeacon._fromMap(b))
            .toList();
        _beaconStreamPool[call.method].add(beacons);
      } else if (method.contains("bluetoothStateListener")) {
        _stateStreamPool[call.method]
            .add(_getBluetoothStateFromKey(call.arguments));
      }
      return;
    });
  }

  /// [Optional] Configuration for the scanner
  void configure({
    /// Should scanner send messages to mqtt protocol. Default [false]
    bool enableMqtt = false,

    /// Time interval[Milliseconds] to send data to IOT cloud.Default is 10 minutes
    /// Valid only if [enableMqtt] is true.
    int timeInterval = 600000,
  }) {
    assert(timeInterval >= 60000);
    this._enableMqtt = enableMqtt;
    this._timeInterval = timeInterval;
  }

  /// Stream which splits the bluetooth adapter state
  Stream<BluetoothState> getBluetoothStateStream() {
    String method = "bluetoothStateListener#${++_stateScannerCount}";
    _stateStreamPool[method] =
        _getNewBluetoothStateStreamController<BluetoothState>(method);
    _channel.invokeMethod(method);
    return _stateStreamPool[method].stream;
  }

  /// Retrives the permission state of the scanner
  Future<BluetoothPermissionState> getPermissionState() async {
    return _getBluetoothPermissionStateFromKey(
        await _channel.invokeMethod<String>("permissionState"));
  }

  /// Asks for location permission from the platfrom
  void requestPermission() {
    _channel.invokeMethod<String>("requestPermission");
  }

  /// Retrives the hardware state of the bluetooth adapter on the device.
  Future<BluetoothState> getBluetoothState() async {
    return _getBluetoothStateFromKey(
        await _channel.invokeMethod<String>("bluetoothState"));
  }

  /// Starts a background scanning session and sends the response to the native side.
  /// [scanId] cannot be a `null` value
  /// [uuids] must have atleast one uuid
  Future<void> scanBeaconsInBackground(
      String scanId,

      /// List of UUIDs that the scanner session should scan.
      List<String> uuids,
      {

      /// [Android] Title of the foreground scanner notification title
      String title = "SnowM Scanner",

      /// [Android] Title of the foreground scanner notification body
      String body = "We are scanning beacons currently.",

      /// Geofence parameters, if this is passed the scanner will only scan when the user gets inside the geofence.
      List<Geofence> geofences = const [],

      /// Custom payload that can be sent to scanner which will be delivered along with the scanned beacons
      Map<String, dynamic> customData = const {},

      /// [only Android] Time interval in milliseconds to scan beacons.Default is 5 seconds
      int backgroundScanPeriod = 5000,

      /// [only Android] Time interval in milliseconds to wait between scans.Default is 10 seconds
      int backgroundBetweenScanPeriod = 10000}) async {
    assert(scanId != null);
    assert(uuids.length > 0);
    assert(await getCurrentScanId() == null);
    await _channel.invokeMethod('scanBeaconBackground', {
      "scanId": scanId,
      "uuids": uuids.map((e) => e.toUpperCase()).toList(),
      "geofences": geofences.map((e) => e.toMap()).toList(),
      "customData": customData,
      "title": title,
      "body": body,
      "backgroundScanPeriod": backgroundScanPeriod,
      "backgroundBetweenScanPeriod": backgroundBetweenScanPeriod
    });
  }

  /// Get the active scanning session id, if no background scanner is active it will return `null`
  Future<String> getCurrentScanId() async {
    return _channel.invokeMethod('getCurrentScanId');
  }

  /// Stops background scanning, the scanner should have been registered in order to stop it.
  Future<void> stopBackgroudScan() async {
    assert(await getCurrentScanId() != null);
    await _channel.invokeMethod('stopBackgroundScan');
  }

  /// Scans and sends the scan response on the foreground ie Dart side
  /// [uuids] must have atleast one uuid
  Stream<List<SnowMBeacon>> scanBeacons({
    /// List of proximity uuid of a beacon to be scanned
    List<String> uuids,

    /// Milliseconds for which the beacons should be cached. Default value is 60 seconds.
    int cacheBeaconsFor = 10000,
  }) {
    String method = "scanBeacons#${++_beaconScannerCount}";
    _beaconStreamPool[method] =
        _getNewBeaconStreamController<List<SnowMBeacon>>(method);
    _channel.invokeMethod(method, {
      "uuids": uuids.map((e) => e.toUpperCase()).toList(),
      "enableMqtt": _enableMqtt,
      "timeInterval": _timeInterval
    });
    Map<String, SnowMBeacon> cachedBeacons = _cachedBeacons[method] ?? {};
    return _beaconStreamPool[method].stream.map((beacons) {
      int currentTime = DateTime.now().millisecondsSinceEpoch;
      beacons = beacons.map((e) {
        e..detectedTime = currentTime;
        cachedBeacons[e.uuid] = e;
        return e;
      }).toList();
      List<SnowMBeacon> freshBeacons = cachedBeacons.values
          .where((b) => (currentTime - b.detectedTime) < cacheBeaconsFor)
          .toList();
      cachedBeacons.clear();
      freshBeacons.forEach((b) {
        cachedBeacons[b.uuid] = b;
      });
      return freshBeacons;
    });
  }

  StreamController<T> _getNewBeaconStreamController<T>(String identifier) {
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelStream", {
        "methodName": identifier,
      });
      _beaconStreamPool.remove(identifier);
    };
    return controller;
  }

  StreamController<T> _getNewBluetoothStateStreamController<T>(
      String identifier) {
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelBluetoothStateListener", {
        "methodName": identifier,
      });
      _beaconStreamPool.remove(identifier);
    };
    return controller;
  }
}

/// Entry point for the scanner to scan beacons
final snowmScanner = _SnowMScanner();
